@@ -26,14 +26,18 @@
 #ifndef _BITLBEE_H
 #define _BITLBEE_H
 
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE /* Stupid GNU :-P */
+#endif
 
 /* Depend on Windows 2000 for now since we need getaddrinfo() */
 #define _WIN32_WINNT 0x0501
 
 #define PACKAGE "BitlBee"
-#define BITLBEE_VERSION "1.2.4"
+#define BITLBEE_VERSION "1.2.7"
 #define VERSION BITLBEE_VERSION
+#define BITLBEE_VER(a,b,c) (((a) << 16) + ((b) << 8) + (c))
+#define BITLBEE_VERSION_CODE BITLBEE_VER(1, 2, 7)
 
 #define MAX_STRING 511
 
@@ -128,7 +132,6 @@
 #include "commands.h"
 #include "account.h"
 #include "nick.h"
-#include "chat.h"
 #include "conf.h"
 #include "log.h"
 #include "ini.h"
@@ -161,6 +164,7 @@ void root_command( irc_t *irc, char *command[] );
 gboolean bitlbee_shutdown( gpointer data, gint fd, b_input_condition cond );
 
 char *set_eval_root_nick( set_t *set, char *new_nick );
+char *set_eval_control_channel( set_t *set, char *new_name );
 
 extern global_t global;
 
