@@ -26,6 +26,7 @@
 #define BITLBEE_CORE
 #include "bitlbee.h"
 #include "ipc.h"
+#include "chat.h"
 
 static void irc_cmd_pass( irc_t *irc, char **cmd )
 {
@@ -446,7 +447,6 @@ static void irc_cmd_away( irc_t *irc, char **cmd )
 {
 	user_t *u = user_find( irc, irc->nick );
 	char *away = cmd[1];
-	account_t *a;
 	
 	if( !u ) return;
 	
@@ -473,13 +473,7 @@ static void irc_cmd_away( irc_t *irc, char **cmd )
 		irc_reply( irc, 305, ":Welcome back" );
 	}
 	
-	for( a = irc->accounts; a; a = a->next )
-	{
-		struct im_connection *ic = a->ic;
-		
-		if( ic && ic->flags & OPT_LOGGED_IN )
-			imc_set_away( ic, u->away );
-	}
+	set_setstr( &irc->set, "away", u->away );
 }
 
 static void irc_cmd_whois( irc_t *irc, char **cmd )
@@ -502,6 +496,8 @@ static void irc_cmd_whois( irc_t *irc, char **cmd )
 			irc_reply( irc, 301, "%s :%s", u->nick, "User is offline" );
 		else if( u->away )
 			irc_reply( irc, 301, "%s :%s", u->nick, u->away );
+		if( u->status_msg )
+			irc_reply( irc, 320, "%s :%s", u->nick, u->status_msg );
 		
 		irc_reply( irc, 318, "%s :End of /WHOIS list", nick );
 	}
