@@ -26,8 +26,9 @@
 
 #include <glib.h>
 
-#include "xmltree.h"
 #include "bitlbee.h"
+#include "md5.h"
+#include "xmltree.h"
 
 extern GSList *jabber_connections;
 
@@ -38,12 +39,13 @@ typedef enum
 	JFLAG_AUTHENTICATED = 2,        /* Set when we're successfully authenticatd. */
 	JFLAG_STREAM_RESTART = 4,       /* Set when we want to restart the stream (after
 	                                   SASL or TLS). */
-	JFLAG_WAIT_SESSION = 8,	        /* Set if we sent a <session> tag and need a reply
+	JFLAG_WANT_SESSION = 8,	        /* Set if the server wants a <session/> tag
 	                                   before we continue. */
-	JFLAG_WAIT_BIND = 16,           /* ... for <bind> tag. */
+	JFLAG_WANT_BIND = 16,           /* ... for <bind> tag. */
 	JFLAG_WANT_TYPING = 32,         /* Set if we ever sent a typing notification, this
 	                                   activates all XEP-85 related code. */
 	JFLAG_XMLCONSOLE = 64,          /* If the user added an xmlconsole buddy. */
+	JFLAG_STARTTLS_DONE = 128,      /* If a plaintext session was converted to TLS. */
 } jabber_flags_t;
 
 typedef enum
@@ -82,7 +84,7 @@ struct jabber_data
 	
 	/* After changing one of these two (or the priority setting), call
 	   presence_send_update() to inform the server about the changes. */
-	struct jabber_away_state *away_state;
+	const struct jabber_away_state *away_state;
 	char *away_message;
 	
 	md5_state_t cached_id_prefix;
@@ -105,6 +107,13 @@ struct jabber_cache_entry
 	jabber_cache_event func;
 };
 
+/* Somewhat messy data structure: We have a hash table with the bare JID as
+   the key and the head of a struct jabber_buddy list as the value. The head
+   is always a bare JID. If the JID has other resources (often the case,
+   except for some transports that don't support multiple resources), those
+   follow. In that case, the bare JID at the beginning doesn't actually
+   refer to a real session and should only be used for operations that
+   support incomplete JIDs. */
 struct jabber_buddy
 {
 	char *bare_jid;
@@ -118,7 +127,7 @@ struct jabber_buddy
 	struct jabber_away_state *away_state;
 	char *away_message;
 	
-	time_t last_act;
+	time_t last_msg;
 	jabber_buddy_flags_t flags;
 	
 	struct jabber_buddy *next;
@@ -206,6 +215,8 @@ typedef enum
 	GET_BUDDY_CREAT = 1,	/* Try to create it, if necessary. */
 	GET_BUDDY_EXACT = 2,	/* Get an exact match (only makes sense with bare JIDs). */
 	GET_BUDDY_FIRST = 4,	/* No selection, simply get the first resource for this JID. */
+	GET_BUDDY_BARE = 8,	/* Get the bare version of the JID (possibly inexistent). */
+	GET_BUDDY_BARE_OK = 16,	/* Allow returning a bare JID if that seems better. */
 } get_buddy_flags_t;
 
 struct jabber_error
