@@ -3,7 +3,7 @@
 *  BitlBee - An IRC to IM gateway                                           *
 *  Jabber module - Misc. stuff                                              *
 *                                                                           *
-*  Copyright 2006 Wilmer van der Gaast <wilmer@gaast.net>                   *
+*  Copyright 2006-2010 Wilmer van der Gaast <wilmer@gaast.net>              
 *                                                                           *
 *  This program is free software; you can redistribute it and/or modify     *
 *  it under the terms of the GNU General Public License as published by     *
@@ -227,10 +227,9 @@ xt_status jabber_cache_handle_packet( struct im_connection *ic, struct xt_node *
 const struct jabber_away_state jabber_away_state_list[] =
 {
 	{ "away",  "Away" },
-	{ "chat",  "Free for Chat" },
+	{ "chat",  "Free for Chat" },   /* WTF actually uses this? */
 	{ "dnd",   "Do not Disturb" },
 	{ "xa",    "Extended Away" },
-	{ "",      "Online" },
 	{ "",      NULL }
 };
 
@@ -238,6 +237,9 @@ const struct jabber_away_state *jabber_away_state_by_code( char *code )
 {
 	int i;
 	
+	if( code == NULL )
+		return NULL;
+	
 	for( i = 0; jabber_away_state_list[i].full_name; i ++ )
 		if( g_strcasecmp( jabber_away_state_list[i].code, code ) == 0 )
 			return jabber_away_state_list + i;
@@ -249,6 +251,9 @@ const struct jabber_away_state *jabber_away_state_by_name( char *name )
 {
 	int i;
 	
+	if( name == NULL )
+		return NULL;
+	
 	for( i = 0; jabber_away_state_list[i].full_name; i ++ )
 		if( g_strcasecmp( jabber_away_state_list[i].full_name, name ) == 0 )
 			return jabber_away_state_list + i;
@@ -339,6 +344,11 @@ struct jabber_buddy *jabber_buddy_add( struct im_connection *ic, char *full_jid_
 	
 	if( ( bud = g_hash_table_lookup( jd->buddies, full_jid ) ) )
 	{
+		/* The first entry is always a bare JID. If there are more, we
+		   should ignore the first one here. */
+		if( bud->next )
+			bud = bud->next;
+		
 		/* If this is a transport buddy or whatever, it can't have more
 		   than one instance, so this is always wrong: */
 		if( s == NULL || bud->resource == NULL )
@@ -373,10 +383,15 @@ struct jabber_buddy *jabber_buddy_add( struct im_connection *ic, char *full_jid_
 	}
 	else
 	{
-		/* Keep in mind that full_jid currently isn't really
-		   a full JID... */
-		new->bare_jid = g_strdup( full_jid );
+		new->full_jid = new->bare_jid = g_strdup( full_jid );
 		g_hash_table_insert( jd->buddies, new->bare_jid, new );
+		
+		if( s )
+		{
+			new->next = g_new0( struct jabber_buddy, 1 );
+			new->next->bare_jid = new->bare_jid;
+			new = new->next;
+		}
 	}
 	
 	if( s )
@@ -402,7 +417,7 @@ struct jabber_buddy *jabber_buddy_add( struct im_connection *ic, char *full_jid_
 struct jabber_buddy *jabber_buddy_by_jid( struct im_connection *ic, char *jid_, get_buddy_flags_t flags )
 {
 	struct jabber_data *jd = ic->proto_data;
-	struct jabber_buddy *bud;
+	struct jabber_buddy *bud, *head;
 	char *s, *jid;
 	
 	jid = jabber_normalize( jid_ );
@@ -414,6 +429,11 @@ struct jabber_buddy *jabber_buddy_by_jid( struct im_connection *ic, char *jid_,
 		*s = 0;
 		if( ( bud = g_hash_table_lookup( jd->buddies, jid ) ) )
 		{
+			bare_exists = 1;
+			
+			if( bud->next )
+				bud = bud->next;
+			
 			/* Just return the first one for this bare JID. */
 			if( flags & GET_BUDDY_FIRST )
 			{
@@ -435,16 +455,9 @@ struct jabber_buddy *jabber_buddy_by_jid( struct im_connection *ic, char *jid_,
 				if( strcmp( bud->resource, s + 1 ) == 0 )
 					break;
 		}
-		else
-		{
-			/* This variable tells the if down here that the bare
-			   JID already exists and we should feel free to add
-			   more resources, if the caller asked for that. */
-			bare_exists = 1;
-		}
 		
 		if( bud == NULL && ( flags & GET_BUDDY_CREAT ) &&
-		    ( !bare_exists || imcb_find_buddy( ic, jid ) ) )
+		    ( bare_exists || imcb_find_buddy( ic, jid ) ) )
 		{
 			*s = '/';
 			bud = jabber_buddy_add( ic, jid );
@@ -458,7 +471,8 @@ struct jabber_buddy *jabber_buddy_by_jid( struct im_connection *ic, char *jid_,
 		struct jabber_buddy *best_prio, *best_time;
 		char *set;
 		
-		bud = g_hash_table_lookup( jd->buddies, jid );
+		head = g_hash_table_lookup( jd->buddies, jid );
+		bud = ( head && head->next ) ? head->next : head;
 		
 		g_free( jid );
 		
@@ -475,22 +489,31 @@ struct jabber_buddy *jabber_buddy_by_jid( struct im_connection *ic, char *jid_,
 		else if( flags & GET_BUDDY_FIRST )
 			/* Looks like the caller doesn't care about details. */
 			return bud;
+		else if( flags & GET_BUDDY_BARE )
+			return head;
 		
 		best_prio = best_time = bud;
 		for( ; bud; bud = bud->next )
 		{
 			if( bud->priority > best_prio->priority )
 				best_prio = bud;
-			if( bud->last_act > best_time->last_act )
+			if( bud->last_msg > best_time->last_msg )
 				best_time = bud;
 		}
 		
 		if( ( set = set_getstr( &ic->acc->set, "resource_select" ) ) == NULL )
 			return NULL;
-		else if( strcmp( set, "activity" ) == 0 )
-			return best_time;
-		else /* if( strcmp( set, "priority" ) == 0 ) */
+		else if( strcmp( set, "priority" ) == 0 )
 			return best_prio;
+		else if( flags & GET_BUDDY_BARE_OK ) /* && strcmp( set, "activity" ) == 0 */
+		{
+			if( best_time->last_msg + set_getint( &ic->acc->set, "activity_timeout" ) >= time( NULL ) )
+				return best_time;
+			else
+				return head;
+		}
+		else
+			return best_time;
 	}
 }
 
@@ -532,7 +555,7 @@ struct jabber_buddy *jabber_buddy_by_ext_jid( struct im_connection *ic, char *ji
 int jabber_buddy_remove( struct im_connection *ic, char *full_jid_ )
 {
 	struct jabber_data *jd = ic->proto_data;
-	struct jabber_buddy *bud, *prev, *bi;
+	struct jabber_buddy *bud, *prev = NULL, *bi;
 	char *s, *full_jid;
 	
 	full_jid = jabber_normalize( full_jid_ );
@@ -542,6 +565,9 @@ int jabber_buddy_remove( struct im_connection *ic, char *full_jid_ )
 	
 	if( ( bud = g_hash_table_lookup( jd->buddies, full_jid ) ) )
 	{
+		if( bud->next )
+			bud = (prev=bud)->next;
+		
 		/* If there's only one item in the list (and if the resource
 		   matches), removing it is simple. (And the hash reference
 		   should be removed too!) */
@@ -549,16 +575,7 @@ int jabber_buddy_remove( struct im_connection *ic, char *full_jid_ )
 		    ( ( s == NULL && bud->resource == NULL ) ||
 		      ( bud->resource && s && strcmp( bud->resource, s + 1 ) == 0 ) ) )
 		{
-			g_hash_table_remove( jd->buddies, bud->bare_jid );
-			g_free( bud->bare_jid );
-			g_free( bud->ext_jid );
-			g_free( bud->full_jid );
-			g_free( bud->away_message );
-			g_free( bud );
-			
-			g_free( full_jid );
-			
-			return 1;
+			return jabber_buddy_remove_bare( ic, full_jid );
 		}
 		else if( s == NULL || bud->resource == NULL )
 		{
@@ -569,7 +586,7 @@ int jabber_buddy_remove( struct im_connection *ic, char *full_jid_ )
 		}
 		else
 		{
-			for( bi = bud, prev = NULL; bi; bi = (prev=bi)->next )
+			for( bi = bud; bi; bi = (prev=bi)->next )
 				if( strcmp( bi->resource, s + 1 ) == 0 )
 					break;
 			
@@ -580,8 +597,7 @@ int jabber_buddy_remove( struct im_connection *ic, char *full_jid_ )
 				if( prev )
 					prev->next = bi->next;
 				else
-					/* The hash table should point at the second
-					   item, because we're removing the first. */
+					/* Don't think this should ever happen anymore. */
 					g_hash_table_replace( jd->buddies, bi->bare_jid, bi->next );
 				
 				g_free( bi->ext_jid );
@@ -650,10 +666,9 @@ int jabber_buddy_remove_bare( struct im_connection *ic, char *bare_jid )
 
 time_t jabber_get_timestamp( struct xt_node *xt )
 {
-	struct tm tp, utc;
 	struct xt_node *c;
-	time_t res, tres;
 	char *s = NULL;
+	struct tm tp;
 	
 	for( c = xt->children; ( c = xt_find_node( c, "x" ) ); c = c->next )
 	{
@@ -671,30 +686,8 @@ time_t jabber_get_timestamp( struct xt_node *xt )
 	
 	tp.tm_year -= 1900;
 	tp.tm_mon --;
-	tp.tm_isdst = -1; /* GRRRRRRRRRRR */
 	
-	res = mktime( &tp );
-	/* Problem is, mktime() just gave us the GMT timestamp for the
-	   given local time... While the given time WAS NOT local. So
-	   we should fix this now.
-	
-	   Now I could choose between messing with environment variables
-	   (kludgy) or using timegm() (not portable)... Or doing the
-	   following, which I actually prefer... */
-	gmtime_r( &res, &utc );
-	utc.tm_isdst = -1; /* Once more: GRRRRRRRRRRRRRRRRRR!!! */
-	if( utc.tm_hour == tp.tm_hour && utc.tm_min == tp.tm_min )
-		/* Sweet! We're in UTC right now... */
-		return res;
-	
-	tres = mktime( &utc );
-	res += res - tres;
-	
-	/* Yes, this is a hack. And it will go wrong around DST changes.
-	   BUT this is more likely to be threadsafe than messing with
-	   environment variables, and possibly more portable... */
-	
-	return res;
+	return mktime_utc( &tp );
 }
 
 struct jabber_error *jabber_error_parse( struct xt_node *node, char *xmlns )
