@@ -34,6 +34,7 @@ static int msn_ns_command( gpointer data, char **cmd, int num_parts );
 static int msn_ns_message( gpointer data, char *msg, int msglen, char **cmd, int num_parts );
 
 static void msn_auth_got_passport_token( struct msn_auth_data *mad );
+static gboolean msn_ns_got_display_name( struct im_connection *ic, char *name );
 
 gboolean msn_ns_connected( gpointer data, gint source, b_input_condition cond )
 {
@@ -228,20 +229,12 @@ static int msn_ns_command( gpointer data, char **cmd, int num_parts )
 				return( 0 );
 			}
 		}
-		else if( num_parts == 7 && strcmp( cmd[2], "OK" ) == 0 )
+		else if( num_parts >= 7 && strcmp( cmd[2], "OK" ) == 0 )
 		{
-			set_t *s;
-			
-			http_decode( cmd[4] );
-			
-			strncpy( ic->displayname, cmd[4], sizeof( ic->displayname ) );
-			ic->displayname[sizeof(ic->displayname)-1] = 0;
-			
-			if( ( s = set_find( &ic->acc->set, "display_name" ) ) )
-			{
-				g_free( s->value );
-				s->value = g_strdup( cmd[4] );
-			}
+			if( num_parts == 7 )
+				msn_ns_got_display_name( ic, cmd[4] );
+			else
+				imcb_log( ic, "Warning: Friendly name in server response was corrupted" );
 			
 			imcb_log( ic, "Authenticated, getting buddy list" );
 			
@@ -419,16 +412,21 @@ static int msn_ns_command( gpointer data, char **cmd, int num_parts )
 		if( !st )
 		{
 			/* FIXME: Warn/Bomb about unknown away state? */
-			st = msn_away_state_list;
+			st = msn_away_state_list + 1;
 		}
 		
-		imcb_buddy_status( ic, cmd[3], OPT_LOGGED_IN |
-		                   ( st->number ? OPT_AWAY : 0 ), st->name, NULL );
+		imcb_buddy_status( ic, cmd[3], OPT_LOGGED_IN | 
+		                   ( st != msn_away_state_list ? OPT_AWAY : 0 ),
+		                   st->name, NULL );
 	}
 	else if( strcmp( cmd[0], "FLN" ) == 0 )
 	{
-		if( cmd[1] )
-			imcb_buddy_status( ic, cmd[1], 0, NULL, NULL );
+		if( cmd[1] == NULL )
+			return 1;
+		
+		imcb_buddy_status( ic, cmd[1], 0, NULL, NULL );
+		
+		msn_sb_start_keepalives( msn_sb_by_handle( ic, cmd[1] ), TRUE );
 	}
 	else if( strcmp( cmd[0], "NLN" ) == 0 )
 	{
@@ -448,11 +446,14 @@ static int msn_ns_command( gpointer data, char **cmd, int num_parts )
 		if( !st )
 		{
 			/* FIXME: Warn/Bomb about unknown away state? */
-			st = msn_away_state_list;
+			st = msn_away_state_list + 1;
 		}
 		
-		imcb_buddy_status( ic, cmd[2], OPT_LOGGED_IN |
-		                   ( st->number ? OPT_AWAY : 0 ), st->name, NULL );
+		imcb_buddy_status( ic, cmd[2], OPT_LOGGED_IN | 
+		                   ( st != msn_away_state_list ? OPT_AWAY : 0 ),
+		                   st->name, NULL );
+		
+		msn_sb_stop_keepalives( msn_sb_by_handle( ic, cmd[2] ) );
 	}
 	else if( strcmp( cmd[0], "RNG" ) == 0 )
 	{
@@ -557,6 +558,9 @@ static int msn_ns_command( gpointer data, char **cmd, int num_parts )
 		imc_logout( ic, allow_reconnect );
 		return( 0 );
 	}
+#if 0
+	/* Discard this one completely for now since I don't care about the ack
+	   and since MSN servers can apparently screw up the formatting. */
 	else if( strcmp( cmd[0], "REA" ) == 0 )
 	{
 		if( num_parts != 5 )
@@ -587,6 +591,7 @@ static int msn_ns_command( gpointer data, char **cmd, int num_parts )
 			imcb_rename_buddy( ic, cmd[3], cmd[4] );
 		}
 	}
+#endif
 	else if( strcmp( cmd[0], "IPG" ) == 0 )
 	{
 		imcb_error( ic, "Received IPG command, we don't handle them yet." );
@@ -662,8 +667,8 @@ static int msn_ns_message( gpointer data, char *msg, int msglen, char **cmd, int
 						imcb_log( ic, "The server is going down for maintenance in %s minutes.", arg1 );
 				}
 				
-				if( arg1 ) g_free( arg1 );
-				if( mtype ) g_free( mtype );
+				g_free( arg1 );
+				g_free( mtype );
 			}
 			else if( g_strncasecmp( ct, "text/x-msmsgsprofile", 20 ) == 0 )
 			{
@@ -671,25 +676,30 @@ static int msn_ns_message( gpointer data, char *msg, int msglen, char **cmd, int
 			}
 			else if( g_strncasecmp( ct, "text/x-msmsgsinitialemailnotification", 37 ) == 0 )
 			{
-				char *inbox = msn_findheader( body, "Inbox-Unread:", blen );
-				char *folders = msn_findheader( body, "Folders-Unread:", blen );
-				
-				if( inbox && folders && set_getbool( &ic->acc->set, "mail_notifications" ) )
+				if( set_getbool( &ic->acc->set, "mail_notifications" ) )
 				{
-					imcb_log( ic, "INBOX contains %s new messages, plus %s messages in other folders.", inbox, folders );
+					char *inbox = msn_findheader( body, "Inbox-Unread:", blen );
+					char *folders = msn_findheader( body, "Folders-Unread:", blen );
+
+					if( inbox && folders )
+						imcb_log( ic, "INBOX contains %s new messages, plus %s messages in other folders.", inbox, folders );
+					
+					g_free( inbox );
+					g_free( folders );
 				}
-				
-				g_free( inbox );
-				g_free( folders );
 			}
 			else if( g_strncasecmp( ct, "text/x-msmsgsemailnotification", 30 ) == 0 )
 			{
-				char *from = msn_findheader( body, "From-Addr:", blen );
-				char *fromname = msn_findheader( body, "From:", blen );
-				
-				if( from && fromname && set_getbool( &ic->acc->set, "mail_notifications" ) )
+				if( set_getbool( &ic->acc->set, "mail_notifications" ) )
 				{
-					imcb_log( ic, "Received an e-mail message from %s <%s>.", fromname, from );
+					char *from = msn_findheader( body, "From-Addr:", blen );
+					char *fromname = msn_findheader( body, "From:", blen );
+					
+					if( from && fromname )
+						imcb_log( ic, "Received an e-mail message from %s <%s>.", fromname, from );
+
+					g_free( from );
+					g_free( fromname );
 				}
 			}
 			else if( g_strncasecmp( ct, "text/x-msmsgsactivemailnotification", 35 ) == 0 )
@@ -731,3 +741,48 @@ static void msn_auth_got_passport_token( struct msn_auth_data *mad )
 		imc_logout( ic, TRUE );
 	}
 }
+
+static gboolean msn_ns_got_display_name( struct im_connection *ic, char *name )
+{
+	set_t *s;
+	
+	if( ( s = set_find( &ic->acc->set, "display_name" ) ) == NULL )
+		return FALSE; /* Shouldn't happen.. */
+	
+	http_decode( name );
+	
+	if( s->value && strcmp( s->value, name ) == 0 )
+	{
+		return TRUE;
+		/* The names match, nothing to worry about. */
+	}
+	else if( s->value != NULL &&
+	         ( strcmp( name, ic->acc->user ) == 0 ||
+	           set_getbool( &ic->acc->set, "local_display_name" ) ) )
+	{
+		/* The server thinks our display name is our e-mail address
+		   which is probably wrong, or the user *wants* us to do this:
+		   Always use the locally set display_name. */
+		return msn_set_display_name( ic, s->value );
+	}
+	else
+	{
+		if( s->value && *s->value )
+			imcb_log( ic, "BitlBee thinks your display name is `%s' but "
+			              "the MSN server says it's `%s'. Using the MSN "
+			              "server's name. Set local_display_name to true "
+			              "to use the local name.", s->value, name );
+		
+		if( g_utf8_validate( name, -1, NULL ) )
+		{
+			g_free( s->value );
+			s->value = g_strdup( name );
+		}
+		else
+		{
+			imcb_log( ic, "Warning: Friendly name in server response was corrupted" );
+		}
+		
+		return TRUE;
+	}
+}
