@@ -1,7 +1,7 @@
   /********************************************************************\
   * BitlBee -- An IRC to other IM-networks gateway                     *
   *                                                                    *
-  * Copyright 2002-2006 Wilmer van der Gaast and others                *
+  * Copyright 2002-2010 Wilmer van der Gaast and others                *
   \********************************************************************/
 
 /*
@@ -32,10 +32,13 @@
 */
 
 #define BITLBEE_CORE
-#include "nogaim.h"
 #include <ctype.h>
 
+#include "nogaim.h"
+#include "chat.h"
+
 static int remove_chat_buddy_silent( struct groupchat *b, const char *handle );
+static char *format_timestamp( irc_t *irc, time_t msg_ts );
 
 GSList *connections;
 
@@ -95,7 +98,19 @@ GList *protocols = NULL;
   
 void register_protocol (struct prpl *p)
 {
-	protocols = g_list_append(protocols, p);
+	int i;
+	gboolean refused = global.conf->protocols != NULL;
+ 
+	for (i = 0; global.conf->protocols && global.conf->protocols[i]; i++)
+ 	{
+ 		if (g_strcasecmp(p->name, global.conf->protocols[i]) == 0)
+			refused = FALSE;
+ 	}
+
+	if (refused)
+		log_message(LOGLVL_WARNING, "Protocol %s disabled\n", p->name);
+	else
+		protocols = g_list_append(protocols, p);
 }
 
 struct prpl *find_protocol(const char *name)
@@ -117,6 +132,7 @@ void nogaim_init()
 	extern void oscar_initmodule();
 	extern void byahoo_initmodule();
 	extern void jabber_initmodule();
+	extern void twitter_initmodule();
 
 #ifdef WITH_MSN
 	msn_initmodule();
@@ -134,6 +150,10 @@ void nogaim_init()
 	jabber_initmodule();
 #endif
 
+#ifdef WITH_TWITTER
+	twitter_initmodule();
+#endif
+
 #ifdef WITH_PLUGINS
 	load_plugins();
 #endif
@@ -265,9 +285,8 @@ void imcb_connected( struct im_connection *ic )
 	ic->keepalive = b_timeout_add( 60000, send_keepalive, ic );
 	ic->flags |= OPT_LOGGED_IN;
 	
-	/* Also necessary when we're not away, at least for some of the
-	   protocols. */
-	imc_set_away( ic, u->away );
+	/* Necessary to send initial presence status, even if we're not away. */
+	imc_away_send_update( ic );
 	
 	/* Apparently we're connected successfully, so reset the
 	   exponential backoff timer. */
@@ -369,7 +388,7 @@ void imcb_ask( struct im_connection *ic, char *msg, void *data,
 
 /* list.c */
 
-void imcb_add_buddy( struct im_connection *ic, char *handle, char *group )
+void imcb_add_buddy( struct im_connection *ic, const char *handle, const char *group )
 {
 	user_t *u;
 	char nick[MAX_NICK_LENGTH+1], *s;
@@ -443,9 +462,10 @@ struct buddy *imcb_find_buddy( struct im_connection *ic, char *handle )
 	return( b );
 }
 
-void imcb_rename_buddy( struct im_connection *ic, char *handle, char *realname )
+void imcb_rename_buddy( struct im_connection *ic, const char *handle, const char *realname )
 {
 	user_t *u = user_findhandle( ic, handle );
+	char *set;
 	
 	if( !u || !realname ) return;
 	
@@ -458,9 +478,26 @@ void imcb_rename_buddy( struct im_connection *ic, char *handle, char *realname )
 		if( ( ic->flags & OPT_LOGGED_IN ) && set_getbool( &ic->irc->set, "display_namechanges" ) )
 			imcb_log( ic, "User `%s' changed name to `%s'", u->nick, u->realname );
 	}
+	
+	set = set_getstr( &ic->acc->set, "nick_source" );
+	if( strcmp( set, "handle" ) != 0 )
+	{
+		char *name = g_strdup( realname );
+		
+		if( strcmp( set, "first_name" ) == 0 )
+		{
+			int i;
+			for( i = 0; name[i] && !isspace( name[i] ); i ++ ) {}
+			name[i] = '\0';
+		}
+		
+		imcb_buddy_nick_hint( ic, handle, name );
+		
+		g_free( name );
+	}
 }
 
-void imcb_remove_buddy( struct im_connection *ic, char *handle, char *group )
+void imcb_remove_buddy( struct im_connection *ic, const char *handle, char *group )
 {
 	user_t *u;
 	
@@ -470,7 +507,7 @@ void imcb_remove_buddy( struct im_connection *ic, char *handle, char *group )
 
 /* Mainly meant for ICQ (and now also for Jabber conferences) to allow IM
    modules to suggest a nickname for a handle. */
-void imcb_buddy_nick_hint( struct im_connection *ic, char *handle, char *nick )
+void imcb_buddy_nick_hint( struct im_connection *ic, const char *handle, const char *nick )
 {
 	user_t *u = user_findhandle( ic, handle );
 	char newnick[MAX_NICK_LENGTH+1], *orig_nick;
@@ -615,13 +652,22 @@ void imcb_buddy_status( struct im_connection *ic, const char *handle, int flags,
 	oa = u->away != NULL;
 	oo = u->online;
 	
-	if( u->away )
-	{
-		g_free( u->away );
-		u->away = NULL;
-	}
+	g_free( u->away );
+	g_free( u->status_msg );
+	u->away = u->status_msg = NULL;
 	
-	if( ( flags & OPT_LOGGED_IN ) && !u->online )
+	if( set_getbool( &ic->irc->set, "show_offline" ) && !u->online )
+	{
+		/* always set users as online */
+		irc_spawn( ic->irc, u );
+		u->online = 1;
+		if( !( flags & OPT_LOGGED_IN ) )
+		{
+			/* set away message if user isn't really online */
+			u->away = g_strdup( "User is offline" );
+		}
+	}
+	else if( ( flags & OPT_LOGGED_IN ) && !u->online )
 	{
 		irc_spawn( ic->irc, u );
 		u->online = 1;
@@ -630,14 +676,30 @@ void imcb_buddy_status( struct im_connection *ic, const char *handle, int flags,
 	{
 		struct groupchat *c;
 		
-		irc_kill( ic->irc, u );
-		u->online = 0;
-		
-		/* Remove him/her from the groupchats to prevent PART messages after he/she QUIT already */
-		for( c = ic->groupchats; c; c = c->next )
-			remove_chat_buddy_silent( c, handle );
+		if( set_getbool( &ic->irc->set, "show_offline" ) )
+		{
+			/* keep offline users in channel and set away message to "offline" */
+			u->away = g_strdup( "User is offline" );
+
+			/* Keep showing him/her in the control channel but not in groupchats. */
+			for( c = ic->groupchats; c; c = c->next )
+			{
+				if( remove_chat_buddy_silent( c, handle ) && c->joined )
+					irc_part( c->ic->irc, u, c->channel );
+			}
+		}
+		else
+		{
+			/* kill offline users */
+			irc_kill( ic->irc, u );
+			u->online = 0;
+
+			/* Remove him/her from the groupchats to prevent PART messages after he/she QUIT already */
+			for( c = ic->groupchats; c; c = c->next )
+				remove_chat_buddy_silent( c, handle );
+		}
 	}
-	
+
 	if( flags & OPT_AWAY )
 	{
 		if( state && message )
@@ -657,13 +719,13 @@ void imcb_buddy_status( struct im_connection *ic, const char *handle, int flags,
 			u->away = g_strdup( "Away" );
 		}
 	}
-	/* else waste_any_state_information_for_now(); */
+	else
+	{
+		u->status_msg = g_strdup( message );
+	}
 	
-	/* LISPy... */
-	if( ( set_getbool( &ic->irc->set, "away_devoice" ) ) &&		/* Don't do a thing when user doesn't want it */
-	    ( u->online ) &&						/* Don't touch offline people */
-	    ( ( ( u->online != oo ) && !u->away ) ||			/* Voice joining people */
-	      ( ( u->online == oo ) && ( oa == !u->away ) ) ) )		/* (De)voice people changing state */
+	/* early if-clause for show_offline even if there is some redundant code here because this isn't LISP but C ;) */
+	if( set_getbool( &ic->irc->set, "show_offline" ) && set_getbool( &ic->irc->set, "away_devoice" ) )
 	{
 		char *from;
 		
@@ -676,16 +738,50 @@ void imcb_buddy_status( struct im_connection *ic, const char *handle, int flags,
 			from = g_strdup_printf( "%s!%s@%s", ic->irc->mynick, ic->irc->mynick,
 			                                    ic->irc->myhost );
 		}
-		irc_write( ic->irc, ":%s MODE %s %cv %s", from, ic->irc->channel,
-		                                          u->away?'-':'+', u->nick );
-		g_free( from );
+
+		/* if we use show_offline, we op online users, voice away users, and devoice/deop offline users */
+		if( flags & OPT_LOGGED_IN )
+		{
+			/* user is "online" (either really online or away) */
+			irc_write( ic->irc, ":%s MODE %s %cv%co %s %s", from, ic->irc->channel,
+			                                          u->away?'+':'-', u->away?'-':'+', u->nick, u->nick );
+		}
+		else
+		{
+			/* user is offline */
+			irc_write( ic->irc, ":%s MODE %s -vo %s %s", from, ic->irc->channel, u->nick, u->nick );
+		}
+	}
+	else
+	{ 
+		/* LISPy... */
+		if( ( set_getbool( &ic->irc->set, "away_devoice" ) ) &&		/* Don't do a thing when user doesn't want it */
+		    ( u->online ) &&						/* Don't touch offline people */
+		    ( ( ( u->online != oo ) && !u->away ) ||			/* Voice joining people */
+		      ( ( u->online == oo ) && ( oa == !u->away ) ) ) )		/* (De)voice people changing state */
+		{
+			char *from;
+
+			if( set_getbool( &ic->irc->set, "simulate_netsplit" ) )
+			{
+				from = g_strdup( ic->irc->myhost );
+			}
+			else
+			{
+				from = g_strdup_printf( "%s!%s@%s", ic->irc->mynick, ic->irc->mynick,
+				                                    ic->irc->myhost );
+			}
+			irc_write( ic->irc, ":%s MODE %s %cv %s", from, ic->irc->channel,
+			                                          u->away?'-':'+', u->nick );
+			g_free( from );
+		}
 	}
 }
 
-void imcb_buddy_msg( struct im_connection *ic, char *handle, char *msg, uint32_t flags, time_t sent_at )
+void imcb_buddy_msg( struct im_connection *ic, const char *handle, char *msg, uint32_t flags, time_t sent_at )
 {
 	irc_t *irc = ic->irc;
-	char *wrapped;
+	char *wrapped, *ts = NULL;
 	user_t *u;
 	
 	u = user_findhandle( ic, handle );
@@ -727,10 +823,19 @@ void imcb_buddy_msg( struct im_connection *ic, char *handle, char *msg, uint32_t
 	if( ( g_strcasecmp( set_getstr( &ic->irc->set, "strip_html" ), "always" ) == 0 ) ||
 	    ( ( ic->flags & OPT_DOES_HTML ) && set_getbool( &ic->irc->set, "strip_html" ) ) )
 		strip_html( msg );
-
+	
+	if( set_getbool( &ic->irc->set, "display_timestamps" ) &&
+	    ( ts = format_timestamp( irc, sent_at ) ) )
+	{
+		char *new = g_strconcat( ts, msg, NULL );
+		g_free( ts );
+		ts = msg = new;
+	}
+	
 	wrapped = word_wrap( msg, 425 );
 	irc_msgfrom( irc, u->nick, wrapped );
 	g_free( wrapped );
+	g_free( ts );
 }
 
 void imcb_buddy_typing( struct im_connection *ic, char *handle, uint32_t flags )
@@ -774,6 +879,35 @@ struct groupchat *imcb_chat_new( struct im_connection *ic, const char *handle )
 	return c;
 }
 
+void imcb_chat_name_hint( struct groupchat *c, const char *name )
+{
+	if( !c->joined )
+	{
+		struct im_connection *ic = c->ic;
+		char stripped[MAX_NICK_LENGTH+1], *full_name;
+		
+		strncpy( stripped, name, MAX_NICK_LENGTH );
+		stripped[MAX_NICK_LENGTH] = '\0';
+		nick_strip( stripped );
+		if( set_getbool( &ic->irc->set, "lcnicks" ) )
+			nick_lc( stripped );
+		
+		full_name = g_strdup_printf( "&%s", stripped );
+		
+		if( stripped[0] &&
+		    nick_cmp( stripped, ic->irc->channel + 1 ) != 0 &&
+		    irc_chat_by_channel( ic->irc, full_name ) == NULL )
+		{
+			g_free( c->channel );
+			c->channel = full_name;
+		}
+		else
+		{
+			g_free( full_name );
+		}
+	}
+}
+
 void imcb_chat_free( struct groupchat *c )
 {
 	struct im_connection *ic = c->ic;
@@ -815,7 +949,7 @@ void imcb_chat_free( struct groupchat *c )
 	}
 }
 
-void imcb_chat_msg( struct groupchat *c, char *who, char *msg, uint32_t flags, time_t sent_at )
+void imcb_chat_msg( struct groupchat *c, const char *who, char *msg, uint32_t flags, time_t sent_at )
 {
 	struct im_connection *ic = c->ic;
 	char *wrapped;
@@ -834,7 +968,11 @@ void imcb_chat_msg( struct groupchat *c, char *who, char *msg, uint32_t flags, t
 	wrapped = word_wrap( msg, 425 );
 	if( c && u )
 	{
-		irc_privmsg( ic->irc, u, "PRIVMSG", c->channel, "", wrapped );
+		char *ts = NULL;
+		if( set_getbool( &ic->irc->set, "display_timestamps" ) )
+			ts = format_timestamp( ic->irc, sent_at );
+		irc_privmsg( ic->irc, u, "PRIVMSG", c->channel, ts ? : "", wrapped );
+		g_free( ts );
 	}
 	else
 	{
@@ -887,7 +1025,7 @@ void imcb_chat_topic( struct groupchat *c, char *who, char *topic, time_t set_at
 
 /* buddy_chat.c */
 
-void imcb_chat_add_buddy( struct groupchat *b, char *handle )
+void imcb_chat_add_buddy( struct groupchat *b, const char *handle )
 {
 	user_t *u = user_findhandle( b->ic, handle );
 	int me = 0;
@@ -922,7 +1060,7 @@ void imcb_chat_add_buddy( struct groupchat *b, char *handle )
 }
 
 /* This function is one BIG hack... :-( EREWRITE */
-void imcb_chat_remove_buddy( struct groupchat *b, char *handle, char *reason )
+void imcb_chat_remove_buddy( struct groupchat *b, const char *handle, const char *reason )
 {
 	user_t *u;
 	int me = 0;
@@ -1028,8 +1166,94 @@ char *set_eval_away_devoice( set_t *set, char *value )
 	return value;
 }
 
+char *set_eval_timezone( set_t *set, char *value )
+{
+	char *s;
+	
+	if( strcmp( value, "local" ) == 0 ||
+	    strcmp( value, "gmt" ) == 0 || strcmp( value, "utc" ) == 0 )
+		return value;
+	
+	/* Otherwise: +/- at the beginning optional, then one or more numbers,
+	   possibly followed by a colon and more numbers. Don't bother bound-
+	   checking them since users are free to shoot themselves in the foot. */
+	s = value;
+	if( *s == '+' || *s == '-' )
+		s ++;
+	
+	/* \d+ */
+	if( !isdigit( *s ) )
+		return SET_INVALID;
+	while( *s && isdigit( *s ) ) s ++;
+	
+	/* EOS? */
+	if( *s == '\0' )
+		return value;
+	
+	/* Otherwise, colon */
+	if( *s != ':' )
+		return SET_INVALID;
+	s ++;
+	
+	/* \d+ */
+	if( !isdigit( *s ) )
+		return SET_INVALID;
+	while( *s && isdigit( *s ) ) s ++;
+	
+	/* EOS */
+	return *s == '\0' ? value : SET_INVALID;
+}
 
-
+static char *format_timestamp( irc_t *irc, time_t msg_ts )
+{
+	time_t now_ts = time( NULL );
+	struct tm now, msg;
+	char *set;
+	
+	/* If the timestamp is <= 0 or less than a minute ago, discard it as
+	   it doesn't seem to add to much useful info and/or might be noise. */
+	if( msg_ts <= 0 || msg_ts > now_ts - 60 )
+		return NULL;
+	
+	set = set_getstr( &irc->set, "timezone" );
+	if( strcmp( set, "local" ) == 0 )
+	{
+		localtime_r( &now_ts, &now );
+		localtime_r( &msg_ts, &msg );
+	}
+	else
+	{
+		int hr, min = 0, sign = 60;
+		
+		if( set[0] == '-' )
+		{
+			sign *= -1;
+			set ++;
+		}
+		else if( set[0] == '+' )
+		{
+			set ++;
+		}
+		
+		if( sscanf( set, "%d:%d", &hr, &min ) >= 1 )
+		{
+			msg_ts += sign * ( hr * 60 + min );
+			now_ts += sign * ( hr * 60 + min );
+		}
+		
+		gmtime_r( &now_ts, &now );
+		gmtime_r( &msg_ts, &msg );
+	}
+	
+	if( msg.tm_year == now.tm_year && msg.tm_yday == now.tm_yday )
+		return g_strdup_printf( "\x02[\x02\x02\x02%02d:%02d:%02d\x02]\x02 ",
+		                        msg.tm_hour, msg.tm_min, msg.tm_sec );
+	else
+		return g_strdup_printf( "\x02[\x02\x02\x02%04d-%02d-%02d "
+		                        "%02d:%02d:%02d\x02]\x02 ",
+		                        msg.tm_year + 1900, msg.tm_mon + 1, msg.tm_mday,
+		                        msg.tm_hour, msg.tm_min, msg.tm_sec );
+}
 
 /* The plan is to not allow straight calls to prpl functions anymore, but do
    them all from some wrappers. We'll start to define some down here: */
@@ -1067,51 +1291,34 @@ int imc_chat_msg( struct groupchat *c, char *msg, int flags )
 	return 1;
 }
 
-static char *imc_away_alias_find( GList *gcm, char *away );
+static char *imc_away_state_find( GList *gcm, char *away, char **message );
 
-int imc_set_away( struct im_connection *ic, char *away )
+int imc_away_send_update( struct im_connection *ic )
 {
-	GList *m, *ms;
-	char *s;
+	char *away, *msg = NULL;
 	
-	if( !away ) away = "";
-	ms = m = ic->acc->prpl->away_states( ic );
+	if( ic->acc->prpl->away_states == NULL ||
+	    ic->acc->prpl->set_away == NULL )
+		return 0;
 	
-	while( m )
+	away = set_getstr( &ic->acc->set, "away" ) ?
+	     : set_getstr( &ic->irc->set, "away" );
+	if( away && *away )
 	{
-		if( *away )
-		{
-			if( g_strncasecmp( m->data, away, strlen( m->data ) ) == 0 )
-				break;
-		}
-		else
-		{
-			if( g_strcasecmp( m->data, "Available" ) == 0 )
-				break;
-			if( g_strcasecmp( m->data, "Online" ) == 0 )
-				break;
-		}
-		m = m->next;
+		GList *m = ic->acc->prpl->away_states( ic );
+		msg = ic->acc->flags & ACC_FLAG_AWAY_MESSAGE ? away : NULL;
+		away = imc_away_state_find( m, away, &msg ) ? : m->data;
+	}
+	else if( ic->acc->flags & ACC_FLAG_STATUS_MESSAGE )
+	{
+		away = NULL;
+		msg = set_getstr( &ic->acc->set, "status" ) ?
+		    : set_getstr( &ic->irc->set, "status" );
 	}
 	
-	if( m )
-	{
-		ic->acc->prpl->set_away( ic, m->data, *away ? away : NULL );
-	}
-	else
-	{
-		s = imc_away_alias_find( ms, away );
-		if( s )
-		{
-			ic->acc->prpl->set_away( ic, s, away );
-			if( set_getbool( &ic->irc->set, "debug" ) )
-				imcb_log( ic, "Setting away state to %s", s );
-		}
-		else
-			ic->acc->prpl->set_away( ic, GAIM_AWAY_CUSTOM, away );
-	}
+	ic->acc->prpl->set_away( ic, away, msg );
 	
-	return( 1 );
+	return 1;
 }
 
 static char *imc_away_alias_list[8][5] =
@@ -1126,16 +1333,33 @@ static char *imc_away_alias_list[8][5] =
 	{ NULL }
 };
 
-static char *imc_away_alias_find( GList *gcm, char *away )
+static char *imc_away_state_find( GList *gcm, char *away, char **message )
 {
 	GList *m;
 	int i, j;
 	
+	for( m = gcm; m; m = m->next )
+		if( g_strncasecmp( m->data, away, strlen( m->data ) ) == 0 )
+		{
+			/* At least the Yahoo! module works better if message
+			   contains no data unless it adds something to what
+			   we have in state already. */
+			if( strlen( m->data ) == strlen( away ) )
+				*message = NULL;
+			
+			return m->data;
+		}
+	
 	for( i = 0; *imc_away_alias_list[i]; i ++ )
 	{
+		int keep_message;
+		
 		for( j = 0; imc_away_alias_list[i][j]; j ++ )
 			if( g_strncasecmp( away, imc_away_alias_list[i][j], strlen( imc_away_alias_list[i][j] ) ) == 0 )
+			{
+				keep_message = strlen( away ) != strlen( imc_away_alias_list[i][j] );
 				break;
+			}
 		
 		if( !imc_away_alias_list[i][j] )	/* If we reach the end, this row */
 			continue;			/* is not what we want. Next!    */
@@ -1143,17 +1367,22 @@ static char *imc_away_alias_find( GList *gcm, char *away )
 		/* Now find an entry in this row which exists in gcm */
 		for( j = 0; imc_away_alias_list[i][j]; j ++ )
 		{
-			m = gcm;
-			while( m )
-			{
+			for( m = gcm; m; m = m->next )
 				if( g_strcasecmp( imc_away_alias_list[i][j], m->data ) == 0 )
-					return( imc_away_alias_list[i][j] );
-				m = m->next;
-			}
+				{
+					if( !keep_message )
+						*message = NULL;
+					
+					return imc_away_alias_list[i][j];
+				}
 		}
+		
+		/* No need to look further, apparently this state doesn't
+		   have any good alias for this protocol. */
+		break;
 	}
 	
-	return( NULL );
+	return NULL;
 }
 
 void imc_add_allow( struct im_connection *ic, char *handle )
