@@ -28,6 +28,7 @@
 #include "base64.h"
 #include "arc.h"
 #include "md5.h"
+#include "chat.h"
 
 #if GLIB_CHECK_VERSION(2,8,0)
 #include <glib/gstdio.h>
@@ -494,6 +495,7 @@ static storage_status_t xml_save( irc_t *irc, int overwrite )
 	if( !xml_printf( fd, 0, "</user>\n" ) )
 		goto write_error;
 	
+	fsync( fd );
 	close( fd );
 	
 	path2 = g_strndup( path, strlen( path ) - 1 );
